% Applies a basic phase-vocoder algorithm to Auto-Tune an out-of-tune voice
% recording.
[wave,fs] = audioread('outoftunesinging.mp3');

wave = wave(1:(fs*10), :);
wave = wave(:, 1);
[s, freq, time] = spectrogram(wave, hanning(1000), 500, 1024, 44100, 'yaxis');
y = displaydBSpectrogram(s+0.00001, freq, time);
title('Spectrogram of Original Singing');
saveas(gcf,'OutOfTuneSinging.bmp','bmp');

% Identify peaks in frequency spectrum
peaks = s;
max_peaks = 1:length(s(1,:));
for i = 1:length(s(1,:))
    max_peaks(i) = 1;
    for j = 3:length(s(:,i))-2
        if abs(s(j,i)) > abs(s(j-1,i)) && abs(s(j,i)) > abs(s(j-2,i)) && abs(s(j,i)) > abs(s(j+1,i)) && abs(s(j,i)) > abs(s(j+2,i)) && abs(s(j,i)) > 1
            peaks(j,i) = 1;
        else
            peaks(j,i) = 0;
        end
        if abs(s(max_peaks(i),i)) < abs(s(j,i))
            max_peaks(i) = j;
        end
    end
end
peaks(1,:) = 0;
peaks(2,:) = 0;
peaks(length(s(:,1))-1,:) = 0;
peaks(length(s(:,1)),:) = 0;

% Check max peaks against pitch table
[pitchtable, midpitchtable] = PitchTable();

[time_bound, freq_bound] = size(s);

s_corrected = s;
s_corrected(:,:) = min(abs(s(:)));
shift_factor = max_peaks;
count = 0;
phase_shift = 1;

for i = 1:length(max_peaks)
    % determine shift factor for wave form
    closest_pitch = compareToPitches(freq(max_peaks(i)), pitchtable, midpitchtable);
    shift_factor(i) = closest_pitch / freq(max_peaks(i));
    
    % shift every frequency by this factor
    for j = 1:length(freq)
        index = ClosestPitchToIndex(shift_factor(i) * freq(j), freq);
        if index > 0
            s_corrected(index, i) = s_corrected(index,i) + s(j,i);
        end
    end
    phase_shift = phase_shift * exp(1i*500*(closest_pitch-freq(max_peaks(i))));
    s_corrected(:,i) = s_corrected(:,i) .* phase_shift;
end

y_corrected = displaydBSpectrogram(s_corrected+0.0001, freq, time);
title('Spectrogram of Tuned Singing');
saveas(gcf, 'TunedSinging.bmp', 'bmp');
s_corrected = conj(s_corrected);
corrected = istft(s_corrected,1024,1000,500);
corrected = corrected';
%soundsc(corrected, fs);
audiowrite('tunedsinging.wav',corrected,fs);
