%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% citation:
% Phu Ngoc Le, E. Ambikairajah and E. Choi, "An improved soft threshold 
% method for DCT speech enhancement," 2008 Second International Conference 
% on Communications and Electronics, Hoi an, 2008, pp. 268-271.
% doi: 10.1109/CCE.2008.4578970
% keywords: {discrete cosine transforms;speech enhancement;
% soft threshold method;discrete cosine transform;DCT domain;
% speech enhancement;noise-dominant frames;signal-dominant frames;
% Discrete cosine transforms;Speech enhancement;Background noise;
% Australia;Additive white noise;Communications technology;
% Signal processing;Gaussian noise;Availability;Fourier transforms;
% speech enhancement;DCT;soft threshold},
% URL: http://ieeexplore.ieee.org/stamp/stamp.jsp?tp=&arnumber=4578970&
% isnumber=4578921
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x_after fs] = denoise(x,fs,SNR)

%read in sample sound
%[xori, fs] = audioread(filename);
% [x, fs] = audioread(filename);
%will only use the first channel for testing. in real project, we will do
%both channel
tbegin=0;
%tend=5;% ending time in second
%tend=floor(length(xori(:,1))/fs);
tend=floor(length(x(:,1))/fs);
%xori=xori(fs*tbegin+1:fs*tend+1,1); 
x=x(fs*tbegin+1:fs*tend+1,1);

%add gaussian noise,
%SNR=15; %dB

%x = awgn(xori,SNR,'measured');
%xnoise=x(1:fs*tend)'-xori(1:fs*tend)';
samples=ones(1,numel(x));
sn = awgn(samples,SNR,'measured');
noise=sn-samples;
estSNR = snr(sn,noise); % channel noise snr get.
sigmav=sqrt(var(x)*10^(-estSNR/10));
%plot(x);

%wavelet spectrogram
minfreq=20;
maxfreq=20000;
VPO=10;

[wt,f] = cwt(x,'morse',fs,'FrequencyLimits',[minfreq maxfreq],'VoicesPerOctave',VPO);

wt=wt(:,1:fs*tend);

%soft thresholding on wt

% threshold for determining whether a block is signal- or noise- dominant

lambdas=0.2; % should be 0.2~0.8
lambdan=0.4; % should be 0.2~0.8 and larger than lambdas
H=length(wt(:,1));
L=length(wt(1,:));
fL=fs/100; %frame length - 0.1s per frame
for i=1:H
    
    
    
    
    %TODO: find an appropriate threshold based on value of the whole row.    
    
    
    for j=1:fL:floor(L/fL)*fL
        framepower=1/fL*sum(abs(wt(i,j:j+fL-1)).^2);
        if(framepower>sigmav^2)
            m=lambdas*sigmav*2/(fL+1);
        else
            m=lambdan*sigmav*2/(fL+1);
        end
        newrow=sort(abs(wt(i,j:j+fL-1)));
        % index is the sorted index of |wt(i,j)| for fixed i and sorted j
        for k=0:fL-1
            absvalue = abs(wt(i,j+k));
            index = find(abs((newrow-absvalue)*10000) < 1,1,'first');
            %index=fL;
            wt(i,j+k)=adsth(wt(i,j+k),m,index);
        end
    end
end

h0=[1 1 1 1 1 1 1 2 2 10 50 10 2 2 1 1 1 1 1 1 1]; %1x21 double
h0=h0./norm(h0,2);
wtsm=conv2(wt,h0);


x_after = icwt(wtsm);
%x_raw=icwt(wt);

%soundsc(xori,fs);
%pause(7);
%soundsc(x,fs);
%pause(tend+1);
%soundsc(x_raw,fs);
%pause(tend+1);
%soundsc(x_after,fs);

%snr(x(1:fs*tend)',xnoise(1:fs*tend))
%snr(x_after(1:fs*tend),xnoise(1:fs*tend))

end
%thresholding 
function y=adsth(x,m,idx)
if(abs(x)>m*idx)
    y=x;
else
    %y=0;
    y=(abs(x)-m*idx);
end
end
