%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Short-Time Fourier Transform            %
%               with MATLAB Implementation             %
%                                                      %
% Author: Ph.D. Eng. Hristo Zhivomirov        12/21/13 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [STFT, f, t] = mystft(x, win, hop, nfft, fs)
    % function: [STFT, f, t] = stft(x, win, hop, nfft, fs)
    %
    % Input:
    % x - signal in the time domain
    % win - analysis window function
    % hop - hop size
    % nfft - number of FFT points
    % fs - sampling frequency, Hz
    %
    % Output:
    % STFT - STFT-matrix (only unique points, time 
    %        across columns, frequency across rows)
    % f - frequency vector, Hz
    % t - time vector, s
    % representation of the signal as column-vector
    x = x(:);
    % determination of the signal length 
    xlen = length(x);
    % determination of the window length
    wlen = length(win);
    % stft matrix size estimation and preallocation
    NUP = ceil((1+nfft)/2);     % calculate the number of unique fft points
    L = 1+fix((xlen-wlen)/hop); % calculate the number of signal frames
    STFT = zeros(NUP, L);       % preallocate the stft matrix
    % STFT (via time-localized FFT)
    for l = 0:L-1
        % windowing
        xw = x(1+l*hop : wlen+l*hop).*win;
        
        % FFT
        X = fft(xw, nfft);
        
        % update of the stft matrix
        STFT(:, 1+l) = X(1:NUP);
    end
    % calculation of the time and frequency vectors
    t = (wlen/2:hop:wlen/2+(L-1)*hop)/fs;
    f = (0:NUP-1)*fs/nfft;
    end